@@ -96,6 +96,17 @@ except ImportError:
     debug(traceback.format_exc())
     error("empy was not detected, please install it.", exit=True)
 
+## Fix unicode bug in empy
+## This should be removed once upstream empy is fixed
+## See: https://github.com/ros-infrastructure/bloom/issues/196
+try:
+    em.str = unicode
+    em.Stream.write_old = em.Stream.write
+    em.Stream.write = lambda self, data: em.Stream.write_old(self, data.encode('utf8'))
+except NameError:
+    pass
+## End fix
+
 TEMPLATE_EXTENSION = '.em'
 
 
@@ -302,6 +313,34 @@ def generate_substitutions_from_package(
     data['changelogs'] = changelogs
     # Summarize dependencies
     summarize_dependency_mapping(data, depends, build_depends, resolved_deps)
+
+    def convertToUnicode(obj):
+        if sys.version_info.major == 2:
+            if isinstance(obj, str):
+                return unicode(obj.decode('utf8'))
+            elif isinstance(obj, unicode):
+                return obj
+        else:
+            if isinstance(obj, bytes):
+                return str(obj.decode('utf8'))
+            elif isinstance(obj, str):
+                return obj
+        if isinstance(obj, list):
+            for i, val in enumerate(obj):
+                obj[i] = convertToUnicode(val)
+            return obj
+        elif isinstance(obj, type(None)):
+            return None
+        elif isinstance(obj, tuple):
+            obj_tmp = list(obj)
+            for i, val in enumerate(obj_tmp):
+                obj_tmp[i] = convertToUnicode(obj_tmp[i])
+            return tuple(obj_tmp)
+        raise RuntimeError('need to deal with type %s' % (str(type(obj))))
+
+    for item in data.items():
+        data[item[0]] = convertToUnicode(item[1])
+
     return data
 
 
@@ -344,7 +383,7 @@ def process_template_files(path, subs):
     return __process_template_folder(debian_dir, subs)
 
 
-def match_branches_with_prefix(prefix, get_branches):
+def match_branches_with_prefix(prefix, get_branches, prune=False):
     debug("match_branches_with_prefix(" + str(prefix) + ", " +
           str(get_branches()) + ")")
     branches = []
@@ -355,7 +394,15 @@ def match_branches_with_prefix(prefix, get_branches):
             branch = branch.split('/', 2)[-1]
         if branch.startswith(prefix):
             branches.append(branch)
-    return list(set(branches))
+    branches = list(set(branches))
+    if prune:
+        # Prune listed branches by packages in latest upstream
+        with inbranch('upstream'):
+            pkg_names, version, pkgs_dict = get_package_data('upstream')
+            for branch in branches:
+                if branch.split(prefix)[-1].strip('/') not in pkg_names:
+                    branches.remove(branch)
+    return branches
 
 
 def get_package_from_branch(branch):
@@ -404,6 +451,9 @@ class DebianGenerator(BloomGenerator):
         add('-p', '--prefix', required=True,
             help="branch prefix to match, and from which create debians"
                  " hint: if you want to match 'release/foo' use 'release'")
+        add('-a', '--match-all', default=False, action="store_true",
+            help="match all branches with the given prefix, "
+                 "even if not in current upstream")
         add('--distros', nargs='+', required=False, default=[],
             help='A list of debian (ubuntu) distros to generate for')
         add('--install-prefix', default=None,
@@ -427,7 +477,7 @@ class DebianGenerator(BloomGenerator):
         if args.install_prefix is None:
             self.install_prefix = self.default_install_prefix
         self.prefix = args.prefix
-        self.branches = match_branches_with_prefix(self.prefix, get_branches)
+        self.branches = match_branches_with_prefix(self.prefix, get_branches, prune=not args.match_all)
         if len(self.branches) == 0:
             error(
                 "No packages found, check your --prefix or --src arguments.",
